//  
//   M5stick SPK-Hat web radio
//
//  Arduino 1.8.5
//  Board : "M5stick-C"
//  


#include <M5StickC.h>
#include <WiFi.h>
#include "AudioFileSourceICYStream.h"
#include "AudioFileSourceBuffer.h"
#include "AudioGeneratorMP3.h"
#include "AudioOutputI2S.h"

// Enter your WiFi setup here:
const char *SSID = "********";
const char *PASSWORD = "********";

uint32_t LastTime = 0;
int playflag = 0;

int ledflag = 0;
int btnaflag = 0;
int btnbflag = 0;

const int LED = 10;
const int BTNA = 37;
const int BTNB = 39;

// Srart Station URL
int sflag = 1;
char *URL = "http://sj32.hnux.com/stream?type=http&nocache=3104";
String station = "Smooth Jazz";

AudioGeneratorMP3 *mp3;
AudioFileSourceICYStream *file;
AudioFileSourceBuffer *buff;
AudioOutputI2S *out;

void setup()
{
  Serial.begin(115200);
  pinMode(LED, OUTPUT);
  digitalWrite(LED , HIGH);
  pinMode(BTNA, INPUT);
  pinMode(BTNB, INPUT);

  M5.begin();
  M5.Lcd.setRotation(3);
  M5.Lcd.fillScreen(BLACK);

  M5.Lcd.setTextSize(2);
  M5.Lcd.println("M5Stick");
  M5.Lcd.println("    WebRadio");

  delay(1000);
  Serial.println("Connecting to WiFi");
 M5.Lcd.println("wifi");
  M5.Lcd.println(" Connecting");

  initwifi();
  //  StartPlaying();
  M5.Lcd.fillScreen(BLACK);
  M5.Lcd.setTextSize(2);
  M5.Lcd.setCursor(0, 0, 2);
  M5.Lcd.print("Ready");

  M5.Lcd.setTextSize(1);
  M5.Lcd.setCursor(20, 40, 2);
  M5.Lcd.print("Channel ");
  M5.Lcd.print(sflag);
  M5.Lcd.setCursor(20, 55, 2);
  M5.Lcd.print(station);

}


void loop()
{
  static int lastms = 0;

  if (playflag == 0) {

    if (digitalRead(BTNA) ==  LOW) {
      StartPlaying();
      playflag = 1;
      M5.Lcd.setTextSize(2);
      M5.Lcd.setCursor(0, 0, 2);
      M5.Lcd.print("Playing");
      btnaflag = 0;
    }

    if (digitalRead(BTNB) ==  LOW) {
      sflag = sflag + 1;
      if (sflag > 9) {
        sflag = 1;
      }

      switch (sflag) {
        case 1:
          {
              URL = "http://sj32.hnux.com/stream?type=http&nocache=3104";
              station = "Smooth Jazz";
            break;
          }
        case 2:
          {
            URL = "http://sl32.hnux.com/stream?type=http&nocache=1257";
               station = "Smooth Lounge";
            break;
          }
        case 3:
          {
            URL = "http://sky1.torontocast.com:9041/stream";
            station = "J-POP";
            break;
          }
        case 4:
          {
            URL = "http://sl32.hnux.com/stream?type=http&nocache=1257";
            station = "sl32.hnux.com";
            break;
          }
        case 5:
          {
            URL = "http://sl32.hnux.com/stream?type=http&nocache=1257";
            station = "sl32.hnux.com";
            break;
          }
        default:
          {
            URL = "http://sl32.hnux.com/stream?type=http&nocache=1257";
            station = "sl32.hnux.com";
            break;
          }
      }
      M5.Lcd.setTextSize(1);
      M5.Lcd.setCursor(20, 40, 2);
      M5.Lcd.print("Channel ");
      M5.Lcd.print(sflag);
      M5.Lcd.setCursor(20, 55, 2);
      M5.Lcd.print(station);
      delay (500);
    }
  }

  if (playflag == 1) {

    if (mp3->isRunning()) {
      if (millis() - lastms > 1000) {
        lastms = millis();
        Serial.printf("Running for %d ms...\n", lastms);

        ledflag = ledflag + 1;
        if (ledflag > 1) {
          ledflag = 0;
          digitalWrite(LED , HIGH);
        } else {
          digitalWrite(LED , LOW);
        }

      }
      if (!mp3->loop()) mp3->stop();
    } else {
      Serial.printf("MP3 done\n");
      playflag = 0;
      digitalWrite(LED , HIGH);
      M5.Lcd.setTextSize(2);
      M5.Lcd.setCursor(0, 0, 2);
      M5.Lcd.print("Stop     ");

      //    ESP.restart();
    }
    if (digitalRead(BTNA) ==  LOW) {
      StopPlaying();
      playflag = 0;
      digitalWrite(LED , HIGH);
      M5.Lcd.setTextSize(2);
      M5.Lcd.setCursor(0, 0, 2);
      M5.Lcd.print("Stop     ");
      delay(500);
    }

  }
}

void StartPlaying()
{
  file = new AudioFileSourceICYStream(URL);
  file->RegisterMetadataCB(MDCallback, (void*)"ICY");
  buff = new AudioFileSourceBuffer(file, 2048);
  buff->RegisterStatusCB(StatusCallback, (void*)"buffer");
  out = new AudioOutputI2S(0, 1); // Output to builtInDAC
  out->SetOutputModeMono(true);
//  out->SetGain(0.3);
  out->SetGain(0.5);
  mp3 = new AudioGeneratorMP3();
  mp3->RegisterStatusCB(StatusCallback, (void*)"mp3");
  mp3->begin(buff, out);
}

void StopPlaying()
{

  if (mp3) {
    mp3->stop();
    delete mp3;
    mp3 = NULL;
  }
  if (buff) {
    buff->close();
    delete buff;
    buff = NULL;
  }
  if (file) {
    file->close();
    delete file;
    file = NULL;
  }
}

void initwifi()
{
  WiFi.disconnect();
  WiFi.softAPdisconnect(true);
  WiFi.mode(WIFI_STA);

  WiFi.begin(SSID, PASSWORD);

  // Try forever
  int i = 0;
  while (WiFi.status() != WL_CONNECTED) {
    Serial.println("...Connecting to WiFi");
    delay(1000);
    i = i + 1;
    if (i > 10) {
      ESP.restart();
    }
  }
  Serial.println("Connected");
}

// Called when a metadata event occurs (i.e. an ID3 tag, an ICY block, etc.
void MDCallback(void *cbData, const char *type, bool isUnicode, const char *string)
{
  const char *ptr = reinterpret_cast<const char *>(cbData);
  (void) isUnicode; // Punt this ball for now
  // Note that the type and string may be in PROGMEM, so copy them to RAM for printf
  char s1[32], s2[64];
  strncpy_P(s1, type, sizeof(s1));
  s1[sizeof(s1) - 1] = 0;
  strncpy_P(s2, string, sizeof(s2));
  s2[sizeof(s2) - 1] = 0;
  Serial.printf("METADATA(%s) '%s' = '%s'\n", ptr, s1, s2);
  Serial.flush();
}

// Called when there's a warning or error (like a buffer underflow or decode hiccup)
void StatusCallback(void *cbData, int code, const char *string)
{
  const char *ptr = reinterpret_cast<const char *>(cbData);
  // Note that the string may be in PROGMEM, so copy it to RAM for printf
  char s1[64];
  strncpy_P(s1, string, sizeof(s1));
  s1[sizeof(s1) - 1] = 0;
  Serial.printf("STATUS(%s) '%d' = '%s'\n", ptr, code, s1);
  Serial.flush();
}

